<?php
// $Id: reptag_process.inc,v 1.23.2.5 2007/02/26 21:07:36 profix898 Exp $

require_once (drupal_get_path('module', 'reptag').'/reptag_helper.inc');
require_once (drupal_get_path('module', 'reptag').'/reptag_module.inc');

define('REPTAG_PROCALL',      0);
define('REPTAG_PROCSTATIC',   1);
define('REPTAG_PROCDYNAMIC',  2);

/**
 * Simple (wrapper) API function for text replacements
 * (You can use reptag_replace() to process a piece of text instead
 *  of calling reptag_process() directly)
 *
 * @param $text
 *   The text/content to be processed.
 * @param $procall
 *   By default only static tags are replaced using reptag_replace(),
 *   set $procall = TRUE to use all tags (static + dynamic) instead. 
 */
function reptag_replace($text, $procall = FALSE) {
  global $user;
  
  $content = new StdClass();
  $content->uid = $user->uid;
  $content->text = $text;
  reptag_process($content, array('text'), $procall ? REPTAG_PROCALL : REPTAG_PROCSTATIC);
  
  return $content->text;
}

/**
 * Perform the actual reptag processing
 * (This function does all the filter/replacement stuff)
 *
 * @param $content
 *   An object containing the content (and its context) to be processed.    
 * @param $fields
 *   An array containing all fields (or field pathes) of $content to be processed
 *    e.g. 'fieldA' for $content->fieldA
 *         'fieldA/fieldB' for $content->fieldA->fieldB
 *         'fieldA/fieldB/fieldC' for $content->fieldA->fieldB-fieldC etc.
 *         (this also works when $content is not an object but an array) 
 * @param $proc
 *   An integer to set the processing mode (e.g. REPTAG_PROCALL, ...).
 * @param $modules
 *   An array containing the names of all modules to be used
 *    e.g. array('system.tags', 'node.tags')   
 *
 *   See DEVELOPER.TXT for detailed information  
 */
function reptag_process(&$content, $fields = array('text'), $proc = REPTAG_PROCALL, $modules = NULL) {
  // return immediately on empty $fields array
  if (empty($fields)) {
    return;
  }
  
  // set default user if no uid is provided
  if (!isset($content->uid)) {
    $content->uid = 0; // anonymous user
  }
  
  // roles associated with current node
  $content_roles = _reptag_user_get_roles($content->uid);
  
  // load module tags
  $reptags = _reptag_module_tags($proc, $content_roles, $content, $modules);
  if ($proc == REPTAG_PROCDYNAMIC && empty($reptags)) {
    return;
  }
  
  // load exclude tags
  $exclude_tags = array();
  foreach ($content_roles as $rid) {
    $rid_tags = _reptag_exclude_loadtags($rid);
    $exclude_tags = array_filter(array_merge($exclude_tags, $rid_tags));
  }
  
  // filter exclude_tags from reptags
  $reptags = _reptag_filter_exclude($reptags, $exclude_tags);
  
  // load side-wide and user tags from tables
  if ($proc != REPTAG_PROCDYNAMIC) {
  
    $iid = array(0, $content->uid);
    $reptag_table = _reptag_table_load($iid);
    
    // remove all html tags for plain text roles
    $plainrep_roles = unserialize(variable_get('reptag_plainrep_roles', serialize(array())));
    if (array_intersect($plainrep_roles, $content_roles)) {
      foreach ($reptag_table as $tag => $repl) {
        $reptag_table[$tag] = check_plain(preg_replace('#<[\/\!]*?[^<>]*?>#si', '', $repl));
      }
    }
    
    // filter exclude_tags from reptag_table
    foreach ($exclude_tags as $key) {
      $key = '#\\'.strtoupper(substr($key, 0, -1)).'\$#si'; 
      unset($reptag_table[$key]);
    }
    
  }
  
  // debug
  if (variable_get('reptag_debug', 0)) {
    drupal_set_message('fields: '.nl2br(htmlspecialchars(print_r($fields, TRUE))), 'error');
    drupal_set_message('module reptags: '.nl2br(htmlspecialchars(print_r(array_keys($reptags), TRUE))), 'error');
    drupal_set_message('table reptags: '.nl2br(htmlspecialchars(print_r(array_keys($reptag_table), TRUE))), 'error');
    drupal_set_message('content: '.nl2br(htmlspecialchars(print_r($content, TRUE))), 'error');
  }
  
  // if 'body' field is included add 'teaser' field also
  // empty fields will be skipped below
  if (in_array('body', $fields)) {
    $fields[] = 'teaser';
  }
    
  // iterate over all $fields
  foreach ($fields as $field) {
    
    // point to the text/string to be processed
    $text =& _reptag_content_field($content, $field);
    // only continue with a valid (non-empty) string
    if (!is_string($text) || empty($text)) {
      continue;
    }
    
    // process table tags
    if ($proc != REPTAG_PROCDYNAMIC) {
      $text = preg_replace(array_keys($reptag_table), array_values($reptag_table), $text);
    }
    
    // process module tags
    foreach ($reptags as $module => $tags) {
      $result = _reptag_module_invoke($module, 'process', $text, $tags, $content);
      $text = $result ? $result : preg_replace(array_keys($tags), array_values($tags), $text);
    }
    
  }
}

/**
 * function _reptag_filter_exclude().
 */
function _reptag_filter_exclude($reptags, $exclude_tags) {
  foreach ($reptags as $module => $tags) {
    $matches = array_intersect(array_keys($tags), array_values($exclude_tags));
    foreach ($matches as $key) {
      unset($reptags[$module][$key]);
    }
  }
  
  return $reptags; 
}

/**
 * function _reptag_validate_xml().
 * (check current content being well-formed x(ht)ml)
 */
function _reptag_validate_xml($xml) {
  $version = phpversion();
  if (version_compare($version, '4.3.0', '>=')) {
    if (version_compare($version, '5.0.0', '>=') && class_exists('DOMDocument')) {
      // PHP 5.0.0+
      
      // extract html tags
      preg_match_all('/<(\/{0,1}[\w]+)[^>]*>/', $xml, $matches);
      $matches = $matches[0];
      
$pretag = <<< PRE
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
<head>
<meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
<title></title>
</head>
<body>
PRE;
      
$posttag = <<< POST
</body>
</html>
POST;
      
      $xmlhtml = $pretag.implode('', $matches).$posttag;
      
      $dom = new DOMDocument;
      $status = ini_set('html_errors', FALSE);
      $dom->loadHTML($xmlhtml);
      if (!$dom->validate()) {
        ini_set('html_errors', $status);
        return FALSE;
      }
      ini_set('html_errors', $status);

    }
    else {
      // PHP 4.3.0+
    
      if (!function_exists('xml_parser_create')) {
        return TRUE;
      }
      
      $xmlhtml = '<body>'.$xml.'</body>';
      $xml_parser = xml_parser_create();      
       if (!xml_parse($xml_parser, $xmlhtml)) {
          drupal_set_message('XML error: '.xml_error_string(xml_get_error_code($xml_parser)).' at line '.xml_get_current_line_number($xml_parser), 'error');
          return FALSE;
       }
      xml_parser_free($xml_parser);
    
    }
  }

  return TRUE;
}

?>
